/*****************************************************************************-
 The Klipper Library, By Kirby L. Wallace.  Copyright (c), 1986-1994
             Wallace Information Systems Engineering


   FUNCTION: _EditScreen( cScreenName, lEditMode, lScrUpdate ) --> NIL

 PARAMETERS: cScreenName: file name of input screen form
             lEditMode  : TRUE if editable
                          FALSE if view only
             lScrUpdate : TRUE: Redraw screen form on each file read
                          FALSE: only redraw GETS

DESCRIPTION: _EditScreen() is a general purpose, screen form editor.  See
             TEST.SCR below for an example of the screen form file and it's
             use.

             Note that WHEN and VALID clauses must be specified as code blocks!

    EXAMPLE:

    USE test
    GOTO TOP

    _fscreen('Data Entry Screen Test')
    _EditScreen('test.scr',.t.,.t.)

    RETURN

    // Top of file TEST.SCR

    TOPLEFT: 4,2
    COLOR: w+/r,w+/b
    PREPROC: PreProc()   // good use for these might be to open and close the
    POSTPROC: PostProc() // files to be used during screen edit

    SCREEN BEGIN
    Ŀ
     DATA INPUT SCREEN TEST - EDIT_SCR.PRG from the Klipper Library, Ver 4.2  
    Ĵ
                                                                              
                                                                              
      Character Field: {1}              Integer Field: {2}                    
                                                                              
        Decimal Field: {3}                 Date Field: {4}                    
                                                                              
        Logical Field: {5}                 Short Note: {6}                    
                                                                              
            Long Note: {7}                                                    
                                                                              
         Ŀ         
           Enter Data as necessary. PgDn when Done, ESC to Quit.            
                  
                                                                              
    Ĵ
     Enter Data, PgDn When Done - ESC to Exit              EDIT_SCR.SCR       
    
    SCREEN END

    {1} FIELD:Character   PICTURE:@K    VALID:{||_FIELD[1]!='ZAP'}
    {2} FIELD:Integer     PICTURE:      VALID:{||_FIELD[2]>100}
    {3} FIELD:Decimal     PICTURE:      VALID:{||.t.}
    {4} FIELD:Date        PICTURE:      VALID:{||.T.}
    {5} FIELD:Logical     PICTURE:Y     VALID:{||.T.}
    {6} FIELD:Note        PICTURE:@K!   VALID:{||trim(_FIELD[6])$'YESNO'}
    {7} FIELD:Longnote    PICTURE:@!    WHEN:{||_FIELD[6]='YES'}

    //Bottom of File: TEST.SCR

    For sake of readability, you can specify the WHEN or VALID label, even if
    they are not subsequently used, as in:

    {1} FIELD:Character PICTURE:@K   WHEN:  VALID:{||_FIELD[1]!='fred'}
    {2} FIELD:Integer   PICTURE:     WHEN:  VALID:{||_FIELD[2]>100}
    {3} FIELD:Decimal   PICTURE:     WHEN:  VALID:{||.t.}
    {4} FIELD:Date      PICTURE:     WHEN:  VALID:{||.T.}
    {5} FIELD:Logical   PICTURE:Y    WHEN:  VALID:{||.T.}
    {6} FIELD:Note      PICTURE:@K!  WHEN:  VALID:{||trim(_FIELD[6])$'YESNO'}
    {7} FIELD:Longnote  PICTURE:     WHEN:  VALID:{||!'ZAP'$_FIELD[7]}

******************************************************************************/

FUNCTION _EditScreen( cScreenName, lEditMode, lScrUpdate )
LOCAL    nFHandle
LOCAL    cTopLeft   := ''
LOCAL    nTopRow    := 0
LOCAL    nTopCol    := 0
LOCAL    cColor     := SetColor()
LOCAL    cPreProc   := ''
LOCAL    cPostProc  := ''
LOCAL    cThis_Line := ''
LOCAL    k, j
LOCAL    i := j := k := 0
LOCAL    cField_Name := ''
LOCAL    cField_Pic  := ''
LOCAL    lField_When := .t.
LOCAL    lField_Valid := .t.

LOCAL    nul

LOCAL    aField_Loc := { } // field_label, row, col, field_name, pic,when,valid

LOCAL    bWhenClause
LOCAL    bValidClause

MEMVAR   GetList
MEMVAR   _FIELD

PRIVATE   _FIELD := { }

lEditMode = iif(lEditMode == NIL,.t.,lEditMode)
lScrUpdate = iif(lScrUpdate == NIL,.t.,lScrUpdate)

if file(cScreenName)

    nFHandle = fopen(cScreenName)

    while !_feof(nFHandle)

        cThis_Line = alltrim(_readline(nFHandle))

        // parse comments
        if '//' $ cThis_Line
            cThis_Line = alltrim(_snatch(cThis_Line,1,at('//',cThis_Line)-1))
        endif

        do case

            case empty(cThis_Line) .or. substr(cThis_Line,1,2) = '//'
                // ignore line

            case lower(substr(cThis_Line,1,8)) = 'topleft:'
                // definition of top left screen corner

                cTopLeft = substr(cThis_Line,at('topleft:',lower(cThis_Line))+8)
                nTopRow  = val(_extract(cTopLeft,',',1))
                nTopCol  = val(_extract(cTopLeft,',',2))

            case lower(substr(cThis_Line,1,6)) = 'color:'
                // definition of screen color

                cColor = substr(cThis_Line,at('color:',lower(cThis_Line))+6)

                SetColor(cColor)

            case lower(substr(cThis_Line,1,8)) = 'preproc:'
                // definition of function to call after screen has
                // been painted but BEFORE GETs are read.  Allows
                // for some custom screen coloring and highlighting

                cPreProc = alltrim(substr(cThis_Line,at('preproc:',lower(cThis_Line))+8))

            case lower(substr(cThis_Line,1,9)) = 'postproc:'
                // definition of function to call after READ has been
                // issued

                cPostProc = alltrim(substr(cThis_Line,at('postproc:',lower(cThis_Line))+9))

            case lower(substr(cThis_Line,1,12)) = 'screen begin'
                // switch to screen draw mode until "screen end:"


                dispbegin()

                while .t.

                    cThis_Line := alltrim(_readline(nFHandle))

                    if lower(substr(cThis_Line,1,10)) = 'screen end'
                        exit
                    endif

                    if lScrUpdate
                        @ nTopRow+i,nTopCol say cThis_Line
                    endif

                    i++

                    j = _ChCount('{',cThis_Line)

                    if j > 0

                        for k = 1 to j
                            aadd(aField_Loc,{'',0,0,'','','',''})
                        next

                        for k = 1 to j

                            // extract field label 1 from {1}, 11 from {11}, etc.
                            aField_Loc[len(aField_Loc)-(j-k),1] = ;
                                _snatch(cThis_Line,_nth_occr(cThis_Line,'{',k)+1,_nth_occr(cThis_Line,'}',k)-1)

                            // it's relative ROW  position
                            aField_Loc[len(aField_Loc)-(j-k),2] = nTopRow+i-1

                            // it's relative COL position
                            aField_Loc[len(aField_Loc)-(j-k),3] = nTopCol+_nth_occr(cThis_Line,'{',k)-1

                            // clear the label from the screen
                            if lScrUpdate
                                @ nTopRow+i-1, nTopCol+_nth_occr(cThis_Line,'{',k)-1 say ;
                                    space(_nth_occr(cThis_Line,'}',k)-_nth_occr(cThis_Line,'{',k)+1)
                            endif

                        next

                    endif

                end

                dispend()

                if type(cPreProc) == 'UI'
                    nul = &cPreProc
                endif

            case lower(substr(cThis_Line,1,1)) = '{'

                // definition of a screen variable  - here only after receiving "SCREEN END", we don't
                // want to confuse a screen label with a screen field location definition. "{}" can occur
                // on the beginning of a line, but inside the SCREEN BEGIN/END coordinates it is a position
                // indicator, AFTER it is a field definition

                // extract field label (ie, {1})

                for i = 1 to len(aField_Loc)

                    if aField_Loc[i,1] = _SubUntil(cThis_Line,'{','}')

                        // extract field name
                        aField_Loc[i,4] = _SubUntil(cThis_Line,'FIELD:',' ')

                        if 'PICTURE:' $ cThis_Line
                            // extract picture
                            aField_Loc[i,5] = _SubUntil(cThis_Line,'PICTURE:',' ')
                        else
                            aField_Loc[i,5] = ''
                        endif

                        if empty(aField_Loc[i,5])
                            aField_Loc[i,5] = _to_char('@K') // get's rid of the quote marks
                        endif

                        // extract when clause

                        if 'WHEN:' $ cThis_Line

                            aField_Loc[i,6] = _SubUntil(cThis_Line,'WHEN:',' ')

                            if empty(aField_Loc[i,6])
                                aField_Loc[i,6] = '{|| .T.}'
                            endif

                        else
                            aField_Loc[i,6] = '{|| .T.}'
                        endif

                        // extract valid clause

                        if 'VALID:' $ cThis_Line

                            aField_Loc[i,7] = _SubUntil(cThis_Line,'VALID:',' ')

                            if empty(aField_Loc[i,7])
                                aField_Loc[i,7] = '{|| .T.}'
                            endif

                        else
                            aField_Loc[i,7] = '{|| .T.}'
                        endif

                        // got it; get outta here
                        exit

                    endif

                next
        end

    end

    // file has been read, issue read on gets

    asort(aField_Loc,,,{ |x,y| x[1] < y[1] })


    for i = 1 to len(aField_Loc)

        // update data array

        asize(_FIELD,len(_FIELD)+1)
        _FIELD[i] = FieldGet(FieldPos(aField_Loc[i,4]))

        @ aField_Loc[i,2],aField_Loc[i,3] GET _FIELD[i] PICTURE aField_Loc[i,5]

        // update getlist variables

        bWhenClause  := &(aField_Loc[i,6])
        bValidClause := &(aField_Loc[i,7])

        GetList[i]:PreBlock  := bWhenClause
        GetList[i]:PostBlock := bValidClause
        GetList[i]:Cargo     := 'Klipper.Lib v1.0'

    next

    CLEAR GETS

    if lEditMode

        if _rlock()

            for i = 1 to len(aField_Loc)

                _FIELD[i] = FieldGet(FieldPos(aField_Loc[i,4]))

                @ aField_Loc[i,2],aField_Loc[i,3] GET _FIELD[i] PICTURE aField_Loc[i,5]

                // update getlist variables

                bWhenClause  := &(aField_Loc[i,6])
                bValidClause := &(aField_Loc[i,7])

                GetList[i]:PreBlock  := bWhenClause
                GetList[i]:PostBlock := bValidClause
                GetList[i]:Cargo     := 'Klipper.Lib v1.0'

            next

            _readf()

            if type(cPostProc) == 'UI'
                nul = &cPostProc
            endif

            for i = 1 to len(aField_Loc)

                FieldPut(FieldPos(aField_Loc[i,4]),_FIELD[i])

            next

            UNLOCK

        else
            _Ack('Record is being edited elsewhere - Cannot Edit...')
        endif

    endif

else
    _terminate('INTERNAL ERROR: Call to Undefined Input Screen:'+cScreenName)
endif

RETURN(NIL)

