/*****************************************************************************
 The Klipper Library, By Kirby L. Wallace.  Copyright (c), 1986-1994
             Wallace Information Systems Engineering


   FUNCTION: MAIN.PRG

 PARAMETERS:

DESCRIPTION: Demo program for _LineEditXXXFunctions.

             See _LineEditGet() and LineEditPut() for descriptions.

             In this program you have only a very simple data entry screen
             with one option for editing comments.  Use PgUp/PgDn to move up
             and down through records.  INS for new record, DEL for delete.

             Use F10 for comments.  Ctrl-W to exit comments.

             There is no double checking or "safety" in the comment edit.
             As soon as you press Ctrl-W or Esc, the comment edit is closed
             without question.  Pop-in, do your stuff, get the heck outta
             dodge.  No questions asked.  You'll see.

             If you want to take the time, you can add "safety" nets anywhere
             you want.  You can add a "Save Comments?" or "Are you sure you
             want to do this?" user prompt at any time between the call to
             your text editing routine (memoedit() etc) and the call to
             _LineEditPut().

             The basic usage structure of a _LineEditGet()/_LineEditPut()
             combination is:

             1) Check the size of the comment before attempting to create
                a string with _LineEditSize().  If your comments are known
                to always be less than 64K, then this step is not necessary.

             2) Get the data into a string:  _LineEditGet()

             3) Edit the string, manipulate it, spell check it, print it, fax
                it, send it to your mother, further encrypt it, Prnt-Scrn it,
                DO WHATEVER TO IT....

             4) Put it back: _LineEditPut()


             The function takes care of adding new lines when comments grow,
             and recycling lines left over after comments shrink.


             USE ESC to quit the program.

*****************************************************************************(s0t0b16.67H*/


FUNCTION main()

#include "klipper.ch"
#include "kfile.ch"
#include "inkey.ch"

LOCAL    cNotes
LOCAL    nKeyPress
LOCAL    lEditMode
LOCAL    cOldColor
LOCAL    nSize
LOCAL    i

FIELD    Name, Rank, Serno, Comment

MEMVAR   GetList, cXxx

PUBLIC   cXxx := ""

BEGIN_FILE testdata.dbf

    FLD SerNo,     C, 11
    FLD Name,      C, 30
    FLD Rank,      C, 10
    FLD Memofield, M
    FLD Access,    N, 8
    FLD Add,       N, 8
    FLD Delete,    N, 8

END_FILE testdata.dbf


************************************************************************
* The NOTE file must have:
*
*     1. The same key fields as the database it is linked to
*     2. A char,nn (1024 Max) (40 recommended) (5 min) field for the
*        comment.  This field can be "tuned" for optimization.
*
* The index for this file must be on the key fields of the owner
* database AND STR(SEQUENCE).  See indexing statements below.
************************************************************************

BEGIN_FILE testnote.dbf

    FLD SerNo,    C, 11
    FLD Comment,  C, 80

END_FILE testnote.dbf


***************************************************************

_Prepare(FALSE)

SET SCORE OFF

// test whether the functions properly set and restore EXACT -
// they won't work with it on This is already set on by _Prepare(),
// but I want to SEE it...
SET EXACT ON 

****************************************************************

USE TestData ALIAS TestData NEW EXCLUSIVE
ERASE TESTDATA.NTX
INDEX ON SerNo TO TestData
PACK

****************************************************************

USE TestNote ALIAS TestNote NEW EXCLUSIVE
ERASE TESTNOTE.NTX
INDEX ON SerNo+Substr(Comment,1,4) TO TestNote
PACK

****************************************************************

_FSCREEN('_LineEditGet() Test')

SELECT TestData

GOTO TOP

lEditMode = FALSE

_Box(4,5,maxrow()-4,maxcol()-5)

@ 07,06 say repl(chr(194),maxcol()-11)
@ 07,05 SAY chr(195)
@ 07,maxcol()-5 say chr(180)

for i = 1 to maxrow()-12

    @ 07+i,06 say repl(chr(197),maxcol()-11)
    @ 07+i,05 SAY chr(195)
    @ 07+i,maxcol()-5 say chr(180)

next

@ 07+i,06 say repl(chr(193),maxcol()-11)

while TRUE

    _KeyList('PgUp/Dn,  - Edit, INS/DEL Record, F9 - Check Space, F10 - Comments, ESC - Exit')

    cOldColor = setcolor(iif(eof().or. bof().or. deleted(),'r*+/b','gr+/b'))

    @ 05,07 SAY 'Name  '+iif(deleted(),'*DELETED*','         ')
    @ 05,32 SAY 'Rank  '
    @ 05,43 SAY 'SerNo '
    @ 05,58 say 'Access    +   -'

    @ 06,07 GET TestData->Name
    @ 06,32 GET TestData->Rank
    @ 06,43 GET TestData->SerNo
    @ 06,60 SAY TestData->Access PICTURE '9999'
    @ 06,66 SAY TestData->Add    PICTURE '999'
    @ 06,70 SAY TestData->Delete PICTURE '999'

    SetColor(cOldColor)

    cXxx = _LineEditGet(TestData->SerNo,'TESTNOTE','COMMENT')

    _TextBrowse(08,08,maxrow()-6,maxcol()-09,(cXxx),BOXNONE,,,'GR+/B',TRUE)

    if lEditMode

        _KeyList('Edit Record: PgDn When Done, ESC to Quit Field.')
        _ReadF()

        lEditMode = FALSE
    else
        CLEAR GETS
    endif

    // _Box(09,07,maxrow()-6,maxcol()-08) // clear out old comment

    nKeyPress = inkey(0)

    do case

        case nKeyPress = K_ESC

            EXIT

        case nKeyPress = K_DEL

            if deleted()
                RECALL
            else
                DELETE
            endif

        case nKeyPress = K_INS

            APPEND BLANK

        case nKeyPress = K_PGUP

            SKIP -1

            if bof()
                tone(2000,1)
                GOTO TOP
            endif

        case nKeyPress = K_PGDN

            SKIP

            if eof()
                tone(2000,1)
                SKIP -1
            endif

        case nKeyPress = K_ENTER

            lEditMode = TRUE

        case nKeyPress = K_F9

            _ack('_LineEditAvail() reports '+ltrim(str(_LineEditAvail('TESTNOTE','COMMENT')))+' Bytes storage immediately available.')

        case nKeyPress = K_F10

            nSize = _LineEditSize(TestData->SerNo,'TESTNOTE','COMMENT')

            if _Q_Confirm('Comment is '+ltrim(str(nSize))+' Bytes.  Continue?')

                cNotes = _LineEditGet(TestData->SerNo,'TESTNOTE','COMMENT')

                SET CURSOR ON
                cNotes = memoedit(cNotes,08,08,maxrow()-6,maxcol()-09)
                SET CURSOR OFF

                _LineEditPut(TestData->SerNo,'TESTNOTE','COMMENT',cNotes)

            endif

    endcase

enddo


CLOSE ALL


RETURN(NIL)

